# -*- coding: utf-8 -*-

import logging

from odoo import api, fields, models, _
from odoo.addons.http_routing.models.ir_http import slug, slugify
from odoo.addons.http_routing.models.ir_http import slug as http_slug
from odoo.http import request
import re
from odoo.exceptions import ValidationError

_logger = logging.getLogger(__name__)



class WebsiteRedirect(models.Model):
    _inherit = "website.rewrite"

    def _get_rewrites(self):
        return [('custom', 'Custom'), ('product.template', 'Product'), ('product.public.category', 'Category')]
    _rewrite_selection = lambda self, *args, **kwargs: self._get_rewrites(*args, **kwargs)
    product_has_category = fields.Boolean('Has Category',default=True,help="this field check if a product is their that doesn't have category.this is because if category hirechy with product is enabled on setting and a product does not have a category it return False")
    is_category_parent = fields.Boolean('Is Super Parent',default=True,help="this field check if a category is super parent and does not has parent itself.this is because if category hirecahy is enabled on setting and a category does not have a a double slash(/) init it return False")
    rewrite_val = fields.Selection(
        string='Create URL Rewrite',
        selection=_rewrite_selection,
        help="Name of model for rewrite management. ex : [('model.model', 'Model')]",
        default='custom'
        )
    create_type = fields.Selection(
        string='Type',
        selection=[('custom', 'Custom'), ('system', 'System')],
        help="Rewrite record create from",
        default='custom'
        )
    record_id = fields.Integer(
        string='ID Path (Model)',
        help="ID of rewrite model"
        )

    @api.model
    def create(self, vals):
        redirectObjs = self.search([
            ('url_from', '=', vals.get('url_from')),
            ('url_to','=', vals.get('url_to')),
            ('record_id','=', vals.get('record_id')),
            ('rewrite_val','=', vals.get('rewrite_val'))
        ])
        if redirectObjs:
            return redirectObjs[0]
        res = super(WebsiteRedirect, self).create(vals)
        return res

    def write(self, vals):
        redirectObjs = []
        if 'url_from' in vals and 'url_to' in vals:
            redirectObjs = self.search([
                ('url_from', '=', vals.get('url_from')),
                ('url_to','=', vals.get('url_to')),
                ('record_id','=', self.record_id),
                ('rewrite_val','=', self.rewrite_val)
            ])
        elif 'url_to' in vals:
            redirectObjs = self.search([
                ('url_to','=', vals.get('url_to')),
                ('url_from','=', self.url_from),
                ('record_id','=', self.record_id),
                ('rewrite_val','=', self.rewrite_val)
            ])
        elif 'url_from' in vals:
            redirectObjs = self.search([
                ('url_from','=', vals.get('url_from')),
                ('url_to','=', self.url_to),
                ('record_id','=', self.record_id),
                ('rewrite_val','=', self.rewrite_val)
            ])
        if redirectObjs:
            return True
        res = super(WebsiteRedirect, self).write(vals)
        return res

    def getUrlSuffix(self, value):
        modelname = value._name
        suffix = self.getSuffix(modelname)
        return suffix

    def trackSuffix(self, modelName):
        website_id = request.env['website'].get_current_website()
        use_suffix = website_id.use_suffix
        suffix = request.env['website.rewrite'].sudo().getSuffix(modelName)
        return {'use_suffix':use_suffix, 'suffix':suffix}

    def getSuffix(self, modelname):
        website_id = request.env['website'].get_current_website()
        suffix = ''
        # if modelname == 'product.public.category':
        #     if website_id.suffix_category:
        #         suffix = website_id.suffix_category
        #     else:
        #         suffix = website_id.manual_redirect
        # if modelname == 'product.template':
        #     if website_id.suffix_product:
        #         suffix = website_id.suffix_product
        #     else:
        #         suffix = website_id.manual_redirect
        if modelname == 'product.public.category':
            if website_id.manual_redirect:
                suffix = website_id.manual_redirect
            elif website_id.use_suffix and website_id.suffix_category:
                suffix = website_id.suffix_category
        if modelname == 'product.template':
            if website_id.manual_redirect:
                suffix = website_id.manual_redirect
            elif website_id.use_suffix and website_id.suffix_product:
                suffix = website_id.suffix_product
        return suffix

    def unsetUrlSuffix(self, value):
        website_id = request.env['website'].get_current_website()
        suffix = website_id.suffix_category
        if suffix:
            value = value.replace(suffix, '')
        suffix = website_id.suffix_product
        if suffix:
            value = value.replace(suffix, '')
        suffix = website_id.manual_redirect
        if suffix:
            value = value.replace(suffix, '')
        return value

    def setRewriteUrl(self, data):
        url_from = "/" + data.get('url_from')
        url_to = "/" + data.get('url_to')
        redirectExistObjs = self.search([
            ('url_to','=', url_from),
            ('record_id', '=', data.get('record_id')),
            ('rewrite_val', '=', data.get('rewrite_val'))
        ])
        for redirectExistObj in redirectExistObjs:
            if redirectExistObj.url_to != url_to:
                redirectExistObj.url_to = url_to
        redirectObjs = self.search([('url_from', '=', url_from), ('url_to','=', url_to)],order='id desc',limit=1)
        if not redirectObjs:
            # data['website_id'] = self.env['website'].get_current_website().id
            data['redirect_type'] = '301'
            data['create_type'] = 'system'
            data['url_from'] = url_from
            data['url_to'] = url_to
            data['name'] = url_to
            redirectObjs = self.create(data)
        else:
            # added by zahid to seet product has category and category is parent
            redirectObjs.write({'product_has_category':data.get('product_has_category',False),'is_category_parent':data.get('is_category_parent',False)})
        return redirectObjs

    def getSlugUrlKeyModel(self, value, model):
        res = request.env['product.public.category'].sudo().search([('url_key', '=', value)])
        if not res:
            res = request.env['product.template'].sudo().search([('url_key', '=', value)])
        if res:
            model = res._name
        return model

    def getCategoryUrl(self, categoryObj):
        catUrl = ''
        path = request.httprequest.path
        # if categoryObj:
        while categoryObj.parent_id:
            categoryObj = categoryObj.parent_id
            urlKey = categoryObj.url_key
            catUrl = urlKey + "/" + catUrl
        return catUrl




    def createUrlKey(self, modelObj, fieldsList):
        url_key = []
        for field in fieldsList:
            if field == 'product': field = 'name'
            if hasattr(modelObj, field):
                name = getattr(modelObj, field)
                name = slugify(name or '').strip().strip('-')
                url_key.append(name)
            elif field == 'category':
                if hasattr(modelObj, 'public_categ_ids'):
                    public_categ_ids = getattr(modelObj, 'public_categ_ids')
                    if public_categ_ids:
                         name = getattr(modelObj, 'public_categ_ids')[0].display_name
                         name = slugify(name)
                         url_key.append(name)
                else:
                    field = 'name'
                    if hasattr(modelObj, field):
                         name = getattr(modelObj, field)
                         name = slugify(name or '').strip().strip('-')
                         url_key.append(name)
            elif field == 'supercategory':
                if hasattr(modelObj, 'parent_id'):
                    name = modelObj.parent_id.display_name
                    if name:
                        name = slugify(name)
                        url_key.insert(0,name)
                else:
                    field = 'public_categ_ids'
                    if hasattr(modelObj, field):
                        public_categ_ids = getattr(modelObj, 'public_categ_ids')
                        if public_categ_ids:
                             name = getattr(modelObj, 'public_categ_ids')[0].display_name
                             if name:
                                 name = slugify(name)
                                 url_key.insert(0, name)
        # url_key.append(str(type(getattr(modelObj, 'categ_id'))))
        # items = [_ for _ in dir(modelObj) if not (_.startswith('__') or _.endswith('__'))]
        # raise Exception(items)
        # if 'category' in fieldsList:
        #     if hasattr(modelObj, 'public_categ_ids'):
        #         url_key = [slugify(getattr(modelObj, 'public_categ_ids')[0].display_name)] + url_key
        urlKey = '-'.join(url_key)
        if not urlKey:
            urlKey = '{}-{}'.format(modelObj.id, modelObj._name.replace(".", "-"))
        return urlKey

    def getFieldList(self, pattern, model):
        website_id = self.env['website'].get_current_website()
        if hasattr(model, 'website_id'):
            website_id = model.website_id or self.env['website'].get_current_website()
        modelPattern = website_id.mapped(pattern)[0]
        fieldsList = []
        if modelPattern:
            if pattern == 'pattern_category':
                if "name" in modelPattern:
                    modelPattern = modelPattern.replace("name", "")
                    modelPattern = ','.join([field for field in modelPattern.split(',') if field.strip()])

            fieldsList = modelPattern.split(',')

        return fieldsList

    def createRedirectForRewrite(self, vals, modelObj, modelName, pattern):
        if 'url_key' in vals and vals['url_key']:
            vals['url_key'] = re.sub('[^A-Za-z0-9]+', '-', vals['url_key'])
            oldUrl = modelObj.url_key
            if oldUrl in ['', False, None]:
                oldUrl = slug(modelObj)
            urlTo = vals.get('url_key')
            if urlTo in ['', False, None]:
                fieldsList = self.getFieldList(pattern, modelObj)
                urlKey = self.createUrlKey(modelObj, fieldsList)
                urlTo = urlKey
                vals['url_key'] = urlTo
            if urlTo == oldUrl:
                oldUrl = slug(modelObj)

            redirectData = {
                'rewrite_val':modelName,
                'record_id':modelObj.id,
                'url_from':oldUrl,
                'url_to':urlTo
            }
            if modelName=='product.template':
                redirectData['product_has_category'] = True if modelObj.public_categ_ids else False
                pass
            if modelName=='product.public.category':
                redirectData['is_category_parent'] = True if not modelObj.parent_id else False
                pass
            self.setRewriteUrl(redirectData)
        return vals

    def setSeoUrlKey(self, pattern, modelObjs):
        failedIds = []
        for modelObj in modelObjs:
            fieldsList = self.getFieldList(pattern, modelObj)
            urlKey = self.createUrlKey(modelObj, fieldsList)
            try:
                modelObj.url_key = urlKey
            except ValidationError as e:
                modelObj.url_key = ''
                if modelObj._name == 'product.template':
                    failedIds.append(modelObj.default_code or '')
            finally:
                pass
        return failedIds

    def get_parent_category(self, hrefUrl=''):
        frontEndLang = request.httprequest.cookies.get('frontend_lang')
        website_id = request.env['website'].get_current_website()
        # website_id = request.env['website'].get_current_website()
        useCategoryUrl = website_id.use_category_url
        useServerRewrites = website_id.use_server_rewrites
        if not useCategoryUrl:
            return {"mainCat":False, "use_server_rewrites":useServerRewrites}
        redirectObj = request.env['website.rewrite'].sudo()
        categoryUrlKey = redirectObj.unsetUrlSuffix(hrefUrl)
        categoryUrlKey = ''.join(categoryUrlKey.split('/')[-1:])
        categoryObj = request.env['product.public.category'].sudo().search([('url_key', '=', categoryUrlKey)], limit=1)
        catUrl = ''
        if categoryObj:
            catUrl = categoryObj.url_key
            while categoryObj.parent_id:
                categoryObj = categoryObj.parent_id
                urlKey = categoryObj.url_key
                catUrl = urlKey + "/" + catUrl
        if catUrl == '':
            catUrl = False
        return {"mainCat":catUrl, "use_server_rewrites":useServerRewrites, 'lang':frontEndLang}

class Website(models.Model):
    _inherit = "website"

    use_suffix = fields.Boolean(string="Use Suffix in URL", default=True)
    suffix_product = fields.Char(string="Suffix in Product URL", default=".html")
    suffix_category = fields.Char(string="Suffix in Category URL", default=".html")
    pattern_product = fields.Char(string="Pattern for Product URL Key", default="id,name")
    pattern_category = fields.Char(string="Pattern for Category URL Key", default="id,name")
    use_category_url = fields.Boolean(string="Use Category URL on Product", help="""Enable to append category along with product URL in Website""", default=True)
    use_category_hierarchy = fields.Boolean(string="Use Category hierarchy", help="""Enable to manage category hierarchy option in Website""", default=True)
    use_server_rewrites = fields.Boolean(
            string="Use Web Server Rewrites",
            help="""
            By enabling this feature page key will remove from url
            For Example :
            '/shop/product/catalog-product' => '/catalog-product'
            '/blog/my-blog/post/my-blog-first-post' => '/my-blog-first-post'
            """,
            default=True
        )
    manual_redirects_ids = fields.One2many(comodel_name='manual.redirect', inverse_name='website_id',
                                           string='Manual redirect ids')
    manual_redirect = fields.Char('Manual Redirect')

    def _enumerate_pages(self, query_string=None, force=False):
        res = super(Website, self)._enumerate_pages(query_string, force)
        useServerRewrites = self.use_server_rewrites
        for rec in res:
            if 'loc' in rec:
                if useServerRewrites:
                    if '/shop' in rec['loc']:
                        rec['loc'] = rec['loc'].replace("/shop",'')
                yield rec

    def get_seo_category_url(self, categ):

        self.ensure_one()
        if self.use_server_rewrites:
            url = "/%s%s" % (self.use_category_hierarchy and self.env['website.rewrite'].getCategoryUrl(categ) or '' , http_slug(categ))
            return url
        return '/shop/category/%s' % slug(categ)

    def get_seo_product_url(self, product):
        self.ensure_one()
        if self.use_server_rewrites:
            url = False
            if request and self.use_category_url:
                currentPage = request.httprequest.environ.get('PATH_INFO')
                currentPage = currentPage.replace("/shop/category/", "/")
                public_category_id = False
                if len(product.public_categ_ids) > 0:
                    public_category_id = product.public_categ_ids[0]
                if public_category_id:
                    redirectObj = self.env['website.rewrite'].sudo().get_parent_category(public_category_id.url_key)
                else:
                    redirectObj = self.env['website.rewrite'].sudo().get_parent_category(currentPage)
                if redirectObj and redirectObj.get('mainCat'):
                    url = redirectObj.get('mainCat') + "/"

            category_url = ''
            if self.use_category_url:
                product_url = http_slug(product).replace(".html", "")
                product_categ = request.env['product.template'].sudo().search([('url_key', '=', product_url)], limit=1)
                product_categ_final = ' '.join(product_categ.categ_id.name.split()[1:])
                sub_categ = request.env['product.public.category'].sudo().search([('name', '=', product_categ_final)],
                                                                                 limit=1)
                if sub_categ:
                    if sub_categ.parent_id.url_key:
                        category_url = sub_categ.parent_id.url_key + "/" + sub_categ.url_key + "/"
                    else:
                        category_url = sub_categ.url_key + "/"

                final_url = '/%s%s' % (category_url and category_url or '', http_slug(product))
                product.website_url = final_url
            else:
                final_url = '/%s%s' % (url and url or '', http_slug(product))
                product.website_url = final_url
        else:
            product.website_url = "/shop/%s" % slug(product)

    @api.model
    def update_seo_url_prod_categ(self):
        env = self.env
        try:
            categoryObjs = env['product.public.category'].search([])
            env['website.rewrite'].setSeoUrlKey('pattern_category', categoryObjs)
            templateObjs = env['product.template'].search([])
            env['website.rewrite'].setSeoUrlKey('pattern_product', templateObjs)
        except Exception as e:
            pass

class ManualRedirects(models.TransientModel):
    _name = 'manual.redirect'
    _description = "Manual Redirect"

    manual_redirect = fields.Char('Manual redirect',required=True)
    website_id = fields.Many2one('website')

